@@ -13,7 +13,8 @@ lang2print = {'python': 'Python',
               'c': 'C',
               'cpp': 'C++',
               'all': 'Mixed',
-              'all_nomatlab': 'Mixed w/o Matlab'}
+              'all_nomatlab': 'Mixed w/o Matlab',
+              'fortran': 'Fortran'}
 _default_lang = 'python'
 
 
@@ -104,18 +105,18 @@ def write_rst(fd, k):
     head = '.. _%s_rst:\n\n%s Example' % (k, k)
     fd.write(head + '\n')
     fd.write((len(head) * '=') + '\n\n')
-    for l in source[k]:
-        write_lang(fd, k, l)
+    for lang in source[k]:
+        write_lang(fd, k, lang)
         fd.write('\n')
 
 
-def write_lang(fd, k, l):
-    head = '%s Version' % lang2print[l]
+def write_lang(fd, k, lang):
+    head = '%s Version' % lang2print[lang]
     fd.write(head + '\n')
     fd.write((len(head) * '-') + '\n\n')
-    write_src(fd, k, l)
+    write_src(fd, k, lang)
     fd.write('\n')
-    write_yml(fd, k, l)
+    write_yml(fd, k, lang)
     fd.write('\n')
 
     
@@ -123,8 +124,10 @@ def write_code_line(fd, s, upone=False, language=None,
                     replacements=[]):
     p = os.path.sep + get_rel_path(s, upone=True)
     if replacements:
-        pnew = os.path.join(os.path.dirname(os.path.dirname(__file__)),
-                            'temp', os.path.basename(s))
+        pdir = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'temp')
+        pnew = os.path.join(pdir, os.path.basename(s))
+        if not os.path.isdir(pdir):
+            os.mkdir(pdir)
         with open(s, 'r') as pfd:
             contents = pfd.read()
         for x, y in replacements:
@@ -146,27 +149,27 @@ def write_code_line(fd, s, upone=False, language=None,
     fd.write("\n")
 
 
-def write_src(fd, k, l, upone=False):
+def write_src(fd, k, lang, upone=False):
     fd.write("Model Code:\n\n")
-    if isinstance(source[k][l], list):
-        for s in source[k][l]:
+    if isinstance(source[k][lang], list):
+        for s in source[k][lang]:
             write_code_line(fd, s, upone=upone)
     else:
-        write_code_line(fd, source[k][l], upone=upone)
+        write_code_line(fd, source[k][lang], upone=upone)
 
     
-def write_yml(fd, k, l, upone=False):
+def write_yml(fd, k, lang, upone=False):
     fd.write("Model YAML:\n\n")
     replacements = []
     if k.startswith('timesync'):
         replacements = [('{{TIMESYNC_TSTEP_A}}', '7'),
                         ('{{TIMESYNC_TSTEP_B}}', '1')]
-    if isinstance(yamls[k][l], list):
-        for y in yamls[k][l]:
+    if isinstance(yamls[k][lang], list):
+        for y in yamls[k][lang]:
             write_code_line(fd, y, upone=upone, language='yaml',
                             replacements=replacements)
     else:
-        write_code_line(fd, yamls[k][l], upone=upone, language='yaml',
+        write_code_line(fd, yamls[k][lang], upone=upone, language='yaml',
                         replacements=replacements)
 
         
