@@ -295,20 +295,37 @@ public:
       switch (type_code_) {
       case T_BOOLEAN:
       case T_INTEGER: {
-	va_arg(ap.va, int);
+	if (ap.using_ptrs) {
+	  get_va_list_ptr_cpp(&ap);
+	} else {
+	  va_arg(ap.va, int);
+	}
 	return 1;
       }
       case T_NULL: {
-	va_arg(ap.va, void*);
+	if (ap.using_ptrs) {
+	  get_va_list_ptr_cpp(&ap);
+	} else {
+	  va_arg(ap.va, void*);
+	}
 	return 1;
       }
       case T_NUMBER: {
-	va_arg(ap.va, double);
+	if (ap.using_ptrs) {
+	  get_va_list_ptr_cpp(&ap);
+	} else {
+	  va_arg(ap.va, double);
+	}
 	return 1;
       }
       case T_STRING: {
-	va_arg(ap.va, char*);
-	va_arg(ap.va, size_t);
+	if (ap.using_ptrs) {
+	  get_va_list_ptr_cpp(&ap);
+	  get_va_list_ptr_cpp(&ap);
+	} else {
+	  va_arg(ap.va, char*);
+	  va_arg(ap.va, size_t);
+	}
 	return 2;
       }
       }
@@ -356,6 +373,27 @@ public:
     int* type_code_modifier = const_cast<int*>(&type_code_);
     *type_code_modifier = check_type();
   }
+  /*!
+    @brief Dummy stand-in to allow access to array/object methods.
+    @param[in] i size_t Index where item type should be added.
+    @param[in] x MetaschemaType* Type to insert at index i.
+  */
+  virtual void update_type_element(size_t i, const MetaschemaType* x) {
+    UNUSED(i);
+    UNUSED(x);
+    ygglog_throw_error("MetaschemaType::update_type_element: This method is only valid for array types.");
+  }
+  /*!
+    @brief Dummy stand-in to allow access to array/object methods.
+    @param[in] k const char* Key where item type should be added.
+    @param[in] x MetaschemaType* Type to insert at key k.
+  */
+  virtual void update_type_element(const char* k, const MetaschemaType* x) {
+    // Prevent C4100 warning on windows by referencing param
+    UNUSED(k);
+    UNUSED(x);
+    ygglog_throw_error("MetaschemaType::update_type_element: This method is only valid for object types.");
+  }
   /*!
     @brief Update the instance's use_generic flag.
     @param[in] new_use_generic const bool New flag value.
@@ -431,10 +469,20 @@ public:
    */
   virtual void set_variable_length(bool new_variable_length) {
 #ifdef _WIN32
-    new_variable_length;
+    UNUSED(new_variable_length);
 #endif 
     ygglog_throw_error("MetaschemaType::set_variable_length: Cannot set variable_length for type '%s'.", type_);
   }
+  /*!
+    @brief Set the _in_table private variable.
+    @param[in] new_in_table bool New value.
+   */
+  virtual void set_in_table(bool new_in_table) {
+#ifdef _WIN32
+    UNUSED(new_in_table);
+#endif 
+    ygglog_throw_error("MetaschemaType::set_in_table: Cannot set in_table for type '%s'.", type_);
+  }
   /*!
     @brief Get the number of elements in the type.
     @returns size_t Number of elements (1 for scalar).
@@ -694,7 +742,12 @@ public:
 			 nargs_exp(), *nargs);
     switch (type_code_) {
     case T_BOOLEAN: {
-      int arg = va_arg(ap.va, int);
+      int arg;
+      if (ap.using_ptrs) {
+	arg = ((int*)get_va_list_ptr_cpp(&ap))[0];
+      } else {
+	arg = va_arg(ap.va, int);
+      }
       (*nargs)--;
       if (arg == 0)
 	writer->Bool(false);
@@ -703,26 +756,47 @@ public:
       return true;
     }
     case T_INTEGER: {
-      int arg = va_arg(ap.va, int);
+      int arg;
+      if (ap.using_ptrs) {
+	arg = ((int*)get_va_list_ptr_cpp(&ap))[0];
+      } else {
+	arg = va_arg(ap.va, int);
+      }
       (*nargs)--;
       writer->Int(arg);
       return true;
     }
     case T_NULL: {
-      va_arg(ap.va, void*);
+      if (ap.using_ptrs) {
+	get_va_list_ptr_cpp(&ap);
+      } else {
+	va_arg(ap.va, void*);
+      }
       (*nargs)--;
       writer->Null();
       return true;
     }
     case T_NUMBER: {
-      double arg = va_arg(ap.va, double);
+      double arg;
+      if (ap.using_ptrs) {
+	arg = ((double*)get_va_list_ptr_cpp(&ap))[0];
+      } else {
+	arg = va_arg(ap.va, double);
+      }
       (*nargs)--;
       writer->Double(arg);
       return true;
     }
     case T_STRING: {
-      char* arg = va_arg(ap.va, char*);
-      size_t arg_siz = va_arg(ap.va, size_t);
+      char* arg;
+      size_t arg_siz;
+      if (ap.using_ptrs) {
+	arg = (char*)get_va_list_ptr_cpp(&ap);
+	arg_siz = ((size_t*)get_va_list_ptr_cpp(&ap))[0];
+      } else {
+	arg = va_arg(ap.va, char*);
+	arg_siz = va_arg(ap.va, size_t);
+      }
       (*nargs)--;
       (*nargs)--;
       writer->String(arg, (rapidjson::SizeType)arg_siz);
@@ -743,7 +817,7 @@ public:
    */
   bool encode_data(rapidjson::Writer<rapidjson::StringBuffer> *writer,
 		   size_t *nargs, ...) const {
-    va_list_t ap_s;
+    va_list_t ap_s = init_va_list();
     va_start(ap_s.va, nargs);
     bool out = encode_data(writer, nargs, ap_s);
     va_end(ap_s.va);
@@ -791,7 +865,7 @@ public:
    */
   bool encode_data_wrap(rapidjson::Writer<rapidjson::StringBuffer> *writer,
 			size_t *nargs, ...) const {
-    va_list_t ap_s;
+    va_list_t ap_s = init_va_list();
     va_start(ap_s.va, nargs);
     bool out = encode_data_wrap(writer, nargs, ap_s);
     va_end(ap_s.va);
@@ -917,8 +991,7 @@ public:
       return serialize(buf, buf_siz, allow_realloc,
 		       (YggGeneric*)(gen_arg.obj));
     }
-    va_list_t ap_copy;
-    va_copy(ap_copy.va, ap.va);
+    va_list_t ap_copy = copy_va_list(ap);
     update_from_serialization_args(nargs, ap_copy);
     if (nargs_exp() != *nargs) {
       ygglog_throw_error("MetaschemaType::serialize: %d arguments expected, but %d provided.",
@@ -996,13 +1069,25 @@ public:
       bool *arg;
       bool **p;
       if (allow_realloc) {
-	p = va_arg(ap.va, bool**);
-	arg = (bool*)realloc(*p, sizeof(bool));
+	if (ap.using_ptrs) {
+	  p = (bool**)get_va_list_ptr_ref_cpp(&ap);
+	} else {
+	  p = va_arg(ap.va, bool**);
+	}
+	if (ap.for_fortran) {
+	  arg = *p;
+	} else {
+	  arg = (bool*)realloc(*p, sizeof(bool));
+	}
 	if (arg == NULL)
 	  ygglog_throw_error("MetaschemaType::decode_data: could not realloc bool pointer.");
 	*p = arg;
       } else {
-	arg = va_arg(ap.va, bool*);
+	if (ap.using_ptrs) {
+	  arg = (bool*)get_va_list_ptr_cpp(&ap);
+	} else {
+	  arg = va_arg(ap.va, bool*);
+	}
       }
       (*nargs)--;
       arg[0] = data.GetBool();
@@ -1014,13 +1099,25 @@ public:
       int *arg;
       int **p;
       if (allow_realloc) {
-	p = va_arg(ap.va, int**);
-	arg = (int*)realloc(*p, sizeof(int));
+	if (ap.using_ptrs) {
+	  p = (int**)get_va_list_ptr_ref_cpp(&ap);
+	} else {
+	  p = va_arg(ap.va, int**);
+	}
+	if (ap.for_fortran) {
+	  arg = *p;
+	} else {
+	  arg = (int*)realloc(*p, sizeof(int));
+	}
 	if (arg == NULL)
 	  ygglog_throw_error("MetaschemaType::decode_data: could not realloc int pointer.");
 	*p = arg;
       } else {
-	arg = va_arg(ap.va, int*);
+	if (ap.using_ptrs) {
+	  arg = (int*)get_va_list_ptr_cpp(&ap);
+	} else {
+	  arg = va_arg(ap.va, int*);
+	}
       }
       (*nargs)--;
       arg[0] = data.GetInt();
@@ -1032,13 +1129,25 @@ public:
       void **arg;
       void ***p;
       if (allow_realloc) {
-	p = va_arg(ap.va, void***);
-	arg = (void**)realloc(*p, sizeof(void*));
+	if (ap.using_ptrs) {
+	  p = (void***)get_va_list_ptr_ref_cpp(&ap);
+	} else {
+	  p = va_arg(ap.va, void***);
+	}
+	if (ap.for_fortran) {
+	  arg = *p;
+	} else {
+	  arg = (void**)realloc(*p, sizeof(void*));
+	}
 	if (arg == NULL)
 	  ygglog_throw_error("MetaschemaType::decode_data: could not realloc void* pointer.");
 	*p = arg;
       } else {
-	arg = va_arg(ap.va, void**);
+	if (ap.using_ptrs) {
+	  arg = (void**)get_va_list_ptr_cpp(&ap);
+	} else {
+	  arg = va_arg(ap.va, void**);
+	}
       }
       (*nargs)--;
       arg[0] = NULL;
@@ -1050,13 +1159,25 @@ public:
       double *arg;
       double **p;
       if (allow_realloc) {
-	p = va_arg(ap.va, double**);
-	arg = (double*)realloc(*p, sizeof(double));
+	if (ap.using_ptrs) {
+	  p = (double**)get_va_list_ptr_ref_cpp(&ap);
+	} else {
+	  p = va_arg(ap.va, double**);
+	}
+	if (ap.for_fortran) {
+	  arg = *p;
+	} else {
+	  arg = (double*)realloc(*p, sizeof(double));
+	}
 	if (arg == NULL)
 	  ygglog_throw_error("MetaschemaType::decode_data: could not realloc double pointer.");
 	*p = arg;
       } else {
-	arg = va_arg(ap.va, double*);
+	if (ap.using_ptrs) {
+	  arg = (double*)get_va_list_ptr_cpp(&ap);
+	} else {
+	  arg = va_arg(ap.va, double*);
+	}
       }
       (*nargs)--;
       arg[0] = data.GetDouble();
@@ -1068,13 +1189,26 @@ public:
       char *arg;
       char **p;
       if (allow_realloc) {
-	p = va_arg(ap.va, char**);
+	if (ap.using_ptrs) {
+	  p = (char**)get_va_list_ptr_ref_cpp(&ap);
+	} else {
+	  p = va_arg(ap.va, char**);
+	}
 	arg = *p;
       } else {
-	arg = va_arg(ap.va, char*);
+	if (ap.using_ptrs) {
+	  arg = (char*)get_va_list_ptr_cpp(&ap);
+	} else {
+	  arg = va_arg(ap.va, char*);
+	}
 	p = &arg;
       }
-      size_t *arg_siz = va_arg(ap.va, size_t*);
+      size_t *arg_siz;
+      if (ap.using_ptrs) {
+	arg_siz = (size_t*)get_va_list_ptr_cpp(&ap);
+      } else {
+	arg_siz = va_arg(ap.va, size_t*);
+      }
       (*nargs)--;
       (*nargs)--;
       int ret = copy_to_buffer(data.GetString(), data.GetStringLength(),
@@ -1103,7 +1237,7 @@ public:
    */
   bool decode_data(rapidjson::Value &data, const int allow_realloc,
 		   size_t *nargs, ...) const {
-    va_list_t ap_s;
+    va_list_t ap_s = init_va_list();
     va_start(ap_s.va, nargs);
     bool out = decode_data(data, allow_realloc, nargs, ap_s);
     va_end(ap_s.va);
@@ -1128,7 +1262,7 @@ public:
     bool out;
     size_t i;
     for (i = 0; i < skip_before_.size(); i++) {
-      va_arg(ap.va, void*);
+      va_list_t_skip(&ap, sizeof(void*));
       (*nargs)--;
     }
     if (use_generic()) {
@@ -1138,7 +1272,7 @@ public:
       out = decode_data(data, allow_realloc, nargs, ap);
     }
     for (i = 0; i < skip_after_.size(); i++) {
-      va_arg(ap.va, void*);
+      va_list_t_skip(&ap, sizeof(void*));
       (*nargs)--;
     }
     return out;
@@ -1158,7 +1292,7 @@ public:
    */
   bool decode_data_wrap(rapidjson::Value &data, const int allow_realloc,
 			size_t *nargs, ...) const {
-    va_list_t ap_s;
+    va_list_t ap_s = init_va_list();
     va_start(ap_s.va, nargs);
     bool out = decode_data_wrap(data, allow_realloc, nargs, ap_s);
     va_end(ap_s.va);
@@ -1205,8 +1339,7 @@ public:
       return deserialize(buf, buf_siz, (YggGeneric*)(gen_arg->obj));
     }
     const size_t nargs_orig = *nargs;
-    va_list_t ap_copy;
-    va_copy(ap_copy.va, ap.va);
+    va_list_t ap_copy = copy_va_list(ap);
     update_from_deserialization_args(nargs, ap_copy);
     if (nargs_exp() != *nargs) {
       ygglog_throw_error("MetaschemaType::deserialize: %d arguments expected, but only %d provided.",
@@ -1420,6 +1553,69 @@ void YggGeneric::get_data(char* obj, size_t nelements) const {
   get_data(obj, nelements, true);
 };
 
+void YggGeneric::add_array_element(YggGeneric* x) {
+  if (type->type_code() != T_ARRAY)
+    ygglog_throw_error("YggGeneric::add_array_element: Generic object is not an array.");
+  YggGenericVector* v = (YggGenericVector*)get_data();
+  size_t i = v->size();
+  set_array_element(i, x);
+};
+
+void YggGeneric::set_array_element(size_t i, YggGeneric* x) {
+  if (type->type_code() != T_ARRAY)
+    ygglog_throw_error("YggGeneric::set_array_element: Generic object is not an array.");
+  YggGenericVector* v = (YggGenericVector*)get_data();
+  if (i > v->size()) {
+    ygglog_throw_error("YggGeneric::set_array_element: Cannot set element %lu, there are only %lu elements in the array.",
+		       i, v->size());
+  } else if (i == v->size()) {
+    v->push_back(x->copy());
+  } else {
+    YggGeneric* old = (*v)[i];
+    delete old;
+    (*v)[i] = x->copy();
+  }
+  type->update_type_element(i, x->type);
+};
+
+YggGeneric* YggGeneric::get_array_element(size_t i) {
+  if (type->type_code() != T_ARRAY)
+    ygglog_throw_error("YggGeneric::get_array_element: Generic object is not an array.");
+  YggGenericVector* v = (YggGenericVector*)get_data();
+  if (i >= v->size()) {
+    ygglog_throw_error("YggGeneric::get_array_element: Cannot get element %lu, there are only %lu elements in the array.",
+		       i, v->size());
+  }
+  YggGeneric* out = (*v)[i];
+  return out;
+};
+
+void YggGeneric::set_object_element(const char* k, YggGeneric* x) {
+  if (type->type_code() != T_OBJECT)
+    ygglog_throw_error("YggGeneric::set_object_element: Generic object is not an object.");
+  YggGenericMap* v = (YggGenericMap*)get_data();
+  YggGenericMap::iterator it;
+  it = v->find(k);
+  if (it != v->end()) {
+    delete it->second;
+  }
+  (*v)[k] = x;
+  type->update_type_element(k, x->type);
+};
+
+YggGeneric* YggGeneric::get_object_element(const char* k) {
+  if (type->type_code() != T_OBJECT)
+    ygglog_throw_error("YggGeneric::get_object_element: Generic object is not an object.");
+  YggGenericMap* v = (YggGenericMap*)get_data();
+  YggGenericMap::iterator it;
+  it = v->find(k);
+  if (it == v->end()) {
+    ygglog_throw_error("YggGeneric::get_object_element: Cannot get element for key %s, it does not exist.", k);
+  }
+  YggGeneric* out = it->second;
+  return out;
+};
+
 void* YggGeneric::get_data(const MetaschemaType* exp_type) const {
   if (*type != *exp_type) {
     printf("Wrapped Type:\n");
@@ -1433,11 +1629,11 @@ void* YggGeneric::get_data(const MetaschemaType* exp_type) const {
 
 size_t YggGeneric::get_data_array_size() const {
   if (type->type_code() != T_ARRAY) {
-    ygglog_throw_error("YggGeneric::get_data_array_item: Object is not an array.");
+    ygglog_throw_error("YggGeneric::get_data_array_size: Object is not an array.");
   }
   YggGenericVector* arr = (YggGenericVector*)get_data();
   if (arr == NULL) {
-    ygglog_throw_error("YggGeneric::get_data_array_item: Array is NULL.");
+    ygglog_throw_error("YggGeneric::get_data_array_size: Array is NULL.");
   }
   return arr->size();
 };
@@ -1493,6 +1689,21 @@ void* YggGeneric::get_data_array_item(const size_t i,
   }
 };
 
+size_t YggGeneric::get_nbytes_array_item(const size_t i) const {
+  if (type->type_code() != T_ARRAY) {
+    ygglog_throw_error("YggGeneric::get_nbytes_array_item: Object is not an array.");
+  }
+  YggGenericVector* arr = (YggGenericVector*)get_data();
+  if (arr == NULL) {
+    ygglog_throw_error("YggGeneric::get_nbytes_array_item: Array is NULL.");
+  }
+  if (i > arr->size()) {
+    ygglog_throw_error("YggGeneric::get_nbytes_array_item: Array has %lu elements, but %lu were requested.", arr->size(), i);
+  }
+  YggGeneric* out = (*arr)[i];
+  return out->nbytes;
+};
+
 void YggGeneric::set_data_array_item(const size_t index,
 				     const YggGeneric* value) {
   if (type->type_code() != T_ARRAY) {
@@ -1532,6 +1743,22 @@ void* YggGeneric::get_data_map_item(const char *key,
   }
 };
 
+size_t YggGeneric::get_nbytes_map_item(const char *key) const {
+  if (type->type_code() != T_OBJECT) {
+    ygglog_throw_error("YggGeneric::get_nbytes_map_item: Object is not a map.");
+  }
+  YggGenericMap* map = (YggGenericMap*)get_data();
+  if (map == NULL) {
+    ygglog_throw_error("YggGeneric::get_nbytes_map_item: Map is NULL.");
+  }
+  YggGenericMap::iterator map_it = map->find(key);
+  if (map_it == map->end()) {
+    ygglog_throw_error("YggGeneric::get_nbytes_map_item: Could not located item for key '%s'.", key);
+  }
+  YggGeneric* out = map_it->second;
+  return out->nbytes;
+};
+
 void YggGeneric::set_data_map_item(const char *key,
 				   const YggGeneric* value) {
   if (type->type_code() != T_OBJECT) {
