@@ -12,6 +12,9 @@ from yggdrasil.metaschema.datatypes.PlyMetaschemaType import (
     _index_type, _color_type, _coord_type,
     _index_conv, _color_conv, _coord_conv,
     _index_fmt, _color_fmt, _coord_fmt)
+
+
+# TODO: Add support for groups
     
 
 _schema_file = os.path.join(_schema_dir, 'obj.json')
@@ -232,15 +235,27 @@ class ObjDict(PlyDict):
     def mesh(self):
         r"""list: Vertices for each face in the structure."""
         mesh = []
-        for i in range(self.count_elements('faces')):
+        for f in self['faces']:
             imesh = []
-            for f in self['faces']:
-                for v in f:
-                    imesh += [self['vertices'][v['vertex_index']][k]
-                              for k in ['x', 'y', 'z']]
+            for v in f:
+                imesh.append([self['vertices'][v['vertex_index']][k]
+                              for k in ['x', 'y', 'z']])
             mesh.append(imesh)
         return mesh
 
+    @property
+    def vertex_normals(self):
+        mesh = None
+        if 'normals' in self:
+            mesh = []
+            for f in self['faces']:
+                imesh = []
+                for v in f:
+                    imesh.append([self['normals'][v['normal_index']][k]
+                                  for k in ['i', 'j', 'k']])
+                mesh.append(imesh)
+        return mesh
+
     @classmethod
     def from_shape(cls, shape, d, conversion=1.0):  # pragma: lpy
         r"""Create a ply dictionary from a PlantGL shape and descritizer.
